/*
 * pkcs11 low level testing
 *
 * Copyright (C) 2002  Zetes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pkcs11-lowlevel.h"

int c_GetMechanismInfo()
{
	int errors = 0;
	CK_SLOT_ID slotID1, slotID2;
	CK_MECHANISM_TYPE mechList[100];
	CK_ULONG nMechs = sizeof(mechList), i;
	CK_MECHANISM_INFO mechInfo;

	printf("C_GetMechanismInfo\n");

	printf("  Cat 1\n");

	if (p11->C_Initialize(NULL_PTR) != CKR_OK) {
		printf("    C_Initialize(NULL_PTR) didn't return CKR_OK\n");
		return ++errors;
	}

	if (p11->C_GetMechanismInfo(12345, CKM_MD5, &mechInfo) != CKR_SLOT_ID_INVALID) {
		printf("    C_GetMechanismInfo(1) didn't return CKR_SLOT_ID_INVALID\n");
		errors++;
	}

	slotID1 = getFirstSlot(1);

	if (p11->C_GetMechanismInfo(slotID1, CKM_MD5, &mechInfo) != CKR_OK) {
		printf("    C_GetMechanismInfo(2) didn't return CKR_OK\n");
		errors++;
	}

	slotID2 = getFirstSlot(0);
	if (p11->C_GetMechanismInfo(slotID2, CKM_MD5, &mechInfo) != CKR_TOKEN_NOT_PRESENT) {
		printf("    C_GetMechanismInfo(3) didn't return CKR_TOKEN_NOT_PRESENT\n");
		errors++;
	}

	printf("  Cat 2\n");

	slotID1 = getFirstSlot(1);

	if (p11->C_GetMechanismInfo(slotID1, CKM_MD5, NULL_PTR) != CKR_ARGUMENTS_BAD) {
		printf("    C_GetMechanismInfo(1) didn't return CKR_ARGUMENTS_BAD\n");
		errors++;
	}

	if (p11->C_GetMechanismInfo(slotID1, 12345, &mechInfo) != CKR_MECHANISM_INVALID) {
		printf("    C_GetMechanismInfo(2) didn't return CKR_MECHANISM_INVALID\n");
		errors++;
	}

	if (p11->C_GetMechanismList(slotID1, mechList, &nMechs) != CKR_OK) {
		printf("    C_GetMechanismList(2) didn't return CKR_OK\n");
		errors++;
	}
	for (i = 0; i < nMechs; i++) {
		if (p11->C_GetMechanismInfo(slotID1, mechList[i], &mechInfo) != CKR_OK) {
			printf("    C_GetMechanismInfo(3) didn't return CKR_OK\n");
			errors++;
		}
	}

	if (p11->C_Finalize(NULL_PTR) != CKR_OK) {
		printf("    ERR: C_Finalize(NULL_PTR) didn't return CKR_OK\n");
		return ++errors;
	}
	if (p11->C_GetMechanismInfo(slotID1, CKM_MD5, &mechInfo) != CKR_CRYPTOKI_NOT_INITIALIZED) {
		printf("    C_GetMechanismInfo(4) didn't return CKR_CRYPTOKI_NOT_INITIALIZED\n");
		errors++;
	}


	return errors;
}
